/*
------------------------------------------------------------------------------
 $Revision: 1.1.2.6.2.2 $
 $Date: 2017/01/31 03:31:37 $

 NAME: cxt_etl_init

 TYPE: ucmacro

 DESCRIPTION:
 This macro is used to define global variables and or sas options
 e.g. 
	To log performance statistics user can add following option statement
	OPTIONS FULLSTIMER ;
 
 SAS VERSION:
 9.4

 INPUT:
 
 OUTPUT:                                                                      
 
 CALLED BY (optional): Call this macro from the driver macro of the job
 

 HISTORY:                                                                     
 userid yyyymmdd defectid desc                                                 
 sinshd 20150521 
------------------------------------------------------------------------------ 
 Copyright (c) 2005-2015, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
------------------------------------------------------------------------------  
*/ 
%macro cxt_etl_init;
	
	/*** CXT_VALIDATE_SOURCE *********************************************************
	 Source data validation option, set it to Y or N ;
    **********************************************************************************/
	%global CXT_VALIDATE_SOURCE;
	%let CXT_VALIDATE_SOURCE = Y;

	/*** CXT_VALIDATE_ETAG *************************************************************
	 Option to check if the downloaded file is same as the file that was uploaded to S3
	 validation depends on AWS algorithm to generate the ETag(Hash). 
	 In case AWS changes the algorithm and ETL is failing because of the ETag is not 
	 matching ,then set CXT_VALIDATE_ETAG=N
	************************************************************************************/
	%global  CXT_VALIDATE_ETAG;
	%let CXT_VALIDATE_ETAG=Y;

	/*** CXT_NORM_JOB_STATUS_WAIT_SEC *************************************************
	 Normalization Job takes some time to finish depending on the the range of data.
	 Pre_extract job polls the status of the job in a loop. 
     set the wait time in number of seconds CXT_NORM_JOB_STATUS_WAIT_SEC=5, 
	 the pre_extract job will wait for 5 seconds before polling the status again.
	************************************************************************************/
	%global CXT_NORM_JOB_STATUS_WAIT_SEC;
	%let CXT_NORM_JOB_STATUS_WAIT_SEC=5;

	/*** Explicitly Setting the CXT timezone ******************************************************
	 The below is an example of how a timezone is to be set explicitly. It is especially
	 useful to get the correct time in case of daylight saving.
	 options timezone='America/Atka';
	************************************************************************************/

	/*** CXT HTTP RETRY OPTIONS*********************************************************
	CXT_HTTP_MAX_RETRY_ATTEMPTS - no of time you want to retry proc http after failure
	CXT_HTTP_RETRY_WAIT_SEC - no of seconds to wait before retrying http call
	************************************************************************************/
	%global CXT_HTTP_MAX_RETRY_ATTEMPTS CXT_HTTP_RETRY_WAIT_SEC;
	%let CXT_HTTP_MAX_RETRY_ATTEMPTS=3;
	%let CXT_HTTP_RETRY_WAIT_SEC=20;

	/*** CXT FILE READ OPTIONS*********************************************************
	CXT_FILE_READ_OPTION - options to use in infile statement e.g. maximum length of the record in the input file
	************************************************************************************/
	%global CXT_FILE_READ_OPTION;
	%let CXT_FILE_READ_OPTION=%str(LRECL=60000);

%mend;
