/*
------------------------------------------------------------------------------
 $Revision: 1.1.2.1 $
 $Date: 2016/04/11 09:43:15 $

 NAME: cxt_validate_source

 TYPE: cmacro

 DESCRIPTION: Validates the source tables for the following

 1. Duplicate records
 2. View_sequence_no is null or >= 1000
 3. attribution_start_csa is null or blank

 SAS VERSION: 9.4
                          
 CALLED BY (optional):   %cxt_extract_source.sas                           

 HISTORY:                                                                     
 userid yyyymmdd defectid desc    
 sinsuj 20140807          New
 sinshd 20160411 S1215403 converted to ucmacro 
------------------------------------------------------------------------------ 
 Copyright (c) 2005-2015, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
------------------------------------------------------------------------------  
*/                                                                              
%macro cxt_validate_source(lib_table_nm=,table_nm=) ;

	%if %symexist(MSG_DSET)=0 %then 
    %do;
        %cxt_external_msg;  
    %end;
   
    %if %symexist(RETCODE)= 0 %then 
    %do;
        %global retcode;    /* return code */
        %let retcode=0;
    %end;
    %else %let retcode=0;

	%let orderby_list=;
	%let duplicate_recs=;
	%let mand_col_recs=;
	%let table_nm_cmp = %upcase(%Kscan(&table_nm,2,.));
	
	%if &table_nm_cmp = ATTRIBUTION %then %do;
		%let dup_col_list = attribution_start_csa;
		%let mand_col_list = attribution_seq_no is null or attribution_start_csa is null or attribution_ts is null or campaign is null or session_number is null or type is null;
		%let orderby_list = session_number attribution_seq_no;
	%end;

	%else %if &table_nm_cmp = BASKET %then %do;
		%let dup_col_list = csa_number,view_sequence_no,record_type;
		%let mand_col_list = attribution_start_csa is null or csa_number is null or record_ts is null or record_type is null or session_number is null or view_sequence_no is null ;
	%end;

	%else %if &table_nm_cmp = BASKET_ACTION %then %do;
		%let dup_col_list = csa_number,view_sequence_no,product_id,action,action_timestamp,product_name,product_sku;
		%let mand_col_list = action is null or action_timestamp is null or attribution_start_csa is null or csa_number is null or session_number is null or view_sequence_no is null;
	%end;

	%else %if &table_nm_cmp = BASKET_ITEM %then %do;
		%let dup_col_list = csa_number,view_sequence_no,record_type,product_id,product_name,product_sku;
		%let mand_col_list = attribution_start_csa is null or csa_number is null or record_type is null or session_number is null or view_sequence_no is null; 
	%end;

	%else %if &table_nm_cmp = CUSTOM_ATTRIBUTE %then %do;
		%let mand_col_list = att_dttm is null or attribution_start_csa is null or csa_number is null or custom_att_name is null or custom_att_value is null or session_number is null or view_sequence_no is null;
	%end;

	%else %if &table_nm_cmp = DOCUMENTS %then %do;
		%let dup_col_list = alt_text,clicked_timestamp,csa_number,object_id,object_link,object_name,view_sequence_no,object_selector_path;
		%let mand_col_list = attribution_start_csa is null or clicked_timestamp is null or csa_number is null or object_link is null or session_number is null or view_sequence_no is null;
	%end;

	%else %if &table_nm_cmp = FORM_PATH %then %do;
		%let dup_col_list = attempt_index,change_index,csa_number,event_ts,field_name,field_id,form_name,status,view_sequence_no;
		%let mand_col_list = attempt_index is null or attribution_start_csa is null or change_index is null or csa_number is null or event_ts is null or following_csa_no is null or form_name is null or session_number is null or status is null or value is null or view_sequence_no is null or was_submit is null or  (field_name is null and field_id is null);
	%end;

	%else %if &table_nm_cmp = GOAL %then %do;
		%let dup_col_list = csa_number,goal_name,goal_timestamp,view_sequence_no,goal_group_name;
		%let mand_col_list = attribution_start_csa is null or csa_number is null or goal_name is null or goal_timestamp is null or session_number is null or view_sequence_no is null; 
	%end;

	%else %if &table_nm_cmp = INTERNAL_SEARCH %then %do;
		%let dup_col_list = int_srch_name,search_timestamp,srch_field_name,srch_field_id,srch_page_csa_number,srch_page_view_seq_no;
		%let mand_col_list = attribution_start_csa is null or results_displayed is null or results_displayed_flg is null or session_number is null or int_srch_name is null or phrase is null or search_timestamp is null or (srch_field_name is null and srch_field_id is null) or srch_page_csa_number is null or srch_page_view_seq_no is null;
		%let orderby_list = session_number srch_page_csa_number int_srch_name search_timestamp;
	%end;

	%else %if &table_nm_cmp = MEDIA %then %do;
		%let dup_col_list = csa_number,view_sequence_no,media_url;
		%let mand_col_list = attribution_start_csa is null or csa_number is null or media is null or media_url is null or play_start_ts is null or session_number is null or view_sequence_no is null;
		%let orderby_list = csa_number view_sequence_no media_url media;
 	%end;

	%else %if &table_nm_cmp = MEDIA_HR %then %do;
		%let dup_col_list = csa_number,view_sequence_no,media_url,playhead_start_time;
		%let mand_col_list = csa_number is null or view_sequence_no is null or viewing_time is null or media_time is null or play_end_ts is null or interaction_count is null or media is null or media_url is null or playhead_start_time is null or playhead_stop_time is null; 
		%let orderby_list = csa_number view_sequence_no media_url media;
	%end;

	%else %if &table_nm_cmp = NEW_SESSIONS %then %do;
		%let dup_col_list = session_number;
		%let mand_col_list = browser_type is null or browser_version is null or client_start_ts is null or cookies_enabled is null or ip_address is null or server_start_ts is null or session_number is null or uid is null or returning_visitor_flg is null or session_timeout is null; 
	%end;

	%else %if &table_nm_cmp = PAGE_ERRORS %then %do;
		%let dup_col_list = csa_number,error_desc,view_sequence_no,error_attribute_a,error_attribute_b,error_timestamp;
		%let mand_col_list = attribution_start_csa  is null or csa_number is null or error_desc is null or session_number is null or view_sequence_no is null or error_timestamp is null;
	%end;

	%else %if &table_nm_cmp = PRODUCT %then %do;
		%let dup_col_list = csa_number,view_sequence_no,product_id,product_name,product_sku;
		%let mand_col_list = attribution_start_csa  is null or csa_number  is null or session_number  is null or view_sequence_no  is null or view_timestamp  is null;
	%end;

	%else %if &table_nm_cmp = PROMOTION %then %do;
		%let dup_col_list = csa_number,displayed_timestamp,used_timestamp,record_type,promotion_no,view_sequence_no;
		%let mand_col_list = attribution_start_csa is null or csa_number  is null or displayed_timestamp  is null or promotion  is null or session_number  is null or type  is null or record_type  is null or derived_display_flg  is null or promotion_no  is null or view_sequence_no  is null;
	%end;

	%else %if &table_nm_cmp = SESSIONS %then %do;
		%let dup_col_list = session_number;
		%let mand_col_list = active_session_duration_ms is null or server_last_activity_ts is null or session_duration_ms is null or session_number is null or session_timeout is null;
	%end;

	%else %if &table_nm_cmp = TRANSACTION_STEP %then %do;
		%let dup_col_list = csa_number,step_name,step_path_index,trns_instance_no,trns_name,view_sequence_no;
		%let mand_col_list = attribution_start_csa is null or csa_number is null or following_csa_no is null or session_number is null or step_attempt_index is null or step_name is null or step_path_index is null or step_timestamp is null or trns_instance_no is null or trns_name is null or view_sequence_no is null; 
		%let orderby_list = session_number csa_number view_sequence_no trns_instance_no step_path_index step_name trns_name;
	%end;

	%else %if &table_nm_cmp = VIEWS %then %do;
		%let dup_col_list = csa_number,view_sequence_no;
		%let mand_col_list = alias0 is null or attribution_start_csa is null or csa_number is null or load_timestamp is null or session_number is null or view_sequence_no is null or server_start_ts is null;
		%let orderby_list = session_number load_timestamp attribution_start_csa;
	%end;

	%else %if &table_nm_cmp = VIEWS_HR %then %do;
		%let dup_col_list = csa_number,view_sequence_no;
		%let mand_col_list = csa_number is null or page_active_time is null or page_view_time is null or view_sequence_no is null or session_number is null;
		%let orderby_list = session_number;
	%end;

	/***********************************************************************************/
	/* No Validations required for ERROR table
	/***********************************************************************************/

	%if &table_nm_cmp eq CXT_ERRORS %then 
	%do;
	    %goto ERROR_EXIT;
	%end;

	/***********************************************************************************/
	/* Duplicate and mandatory column checks 
	/***********************************************************************************/

	/*1. Duplicate check*/

	%if &table_nm_cmp ne CUSTOM_ATTRIBUTE %then %do;

		proc sql noprint;

			select count(1) 
			into :duplicate_recs
			from
			(
			    select &dup_col_list. 
					    ,count(1)
			    from   &lib_table_nm.
			    group by &dup_col_list. 
				having count(1) > 1
			);
		quit;

		%if &duplicate_recs. > 0 %then 
		%do;

			%let dup_col_list_cr = %SYSFUNC(tranwrd(%quote(&dup_col_list.),%str(,),%str()));
			%let errmsg = %sysfunc(sasmsg(&msg_dset,_cxavldt01_error,noquote,&duplicate_recs,&table_nm_cmp,&dup_col_list_cr));
			
			/* Eliminate Duplicates */

			proc sort data = &lib_table_nm. nodupkey;
				by &dup_col_list_cr;
			quit;

			/* Re Order-By as required by staging*/

			%if &orderby_list ne %str() %then %do; 
				proc sort data = &lib_table_nm.;
					by &orderby_list;
				quit;
			%end;

			proc sql noprint;

				select load_no
					   ,event_no 
				into   :v_load_no
					  ,:v_event_no
				from &work_load_hist;

				select job_id into :v_job_id	
				from &norm_hist
				where norm_id = (select max(norm_id) from &norm_hist);

				select process_dttm into :load_dttm
		        from &etl_signal;
		         
			quit;

			proc sql noprint;
				insert into &cxt_error_log
						(	
							event_no             
							,load_no              
							,error_dttm           
							,ace_rest_api_job_id  
							,error_source_cd      
							,location_txt         
							,error_type_txt       
							,error_txt            
							,load_dttm            
						)
				values
						(
							&v_event_no
							,&v_load_no
							,"&etls_startTime"dt  
							,"&v_job_id"
							,'ETL'
							,"&etls_jobName."
							,'Duplicate Data'
							,"&errmsg."
							,"&load_dttm"dt
						);
			quit;

			%if &SQLRC > 4 %then %do;
				%let retcode = 1;
				%let errmsg = %sysfunc(sasmsg(&msg_dset,_cxasqlchk102_error,noquote)); 
				%GOTO ERROR_EXIT;
			%end;

		%end;

	%end;

	/*2. Mandatory column check*/

	proc sql noprint;
	
		select count(1) 
		into :mand_col_recs
		from
		(
		    select '1'
		    from   &lib_table_nm.
			where &mand_col_list.
		);		

	quit;

	%if &mand_col_recs. > 0 %then 
	%do;
	    %let retcode=1;
		%let errmsg = %sysfunc(sasmsg(&msg_dset,_cxavldt04_error,noquote,&table_nm_cmp,&mand_col_list,&mand_col_recs));
	    %goto ERROR_EXIT;
	%end;
	
	%******************************************************************************;
    %* Error exit point.                                                          ;
    %******************************************************************************;

	%ERROR_EXIT:

    %if &retcode=1 %then 
    %do;
        %let typemsg = %sysfunc(sasmsg(&msg_dset
                             ,_cxaerrmsg_label
                             ,noquote)); 
        %put &typemsg &errmsg;

        %cxt_job_checks(job_section=END
                       ,job_status=UNSUCCESSFUL);
    %end;
	
%mend;
